import "../../style/steps-tree.css"
import { StepInfoNode } from "../../model/tactic/Tactic"
import BendableArrow from "../arrows/BendableArrow"
import { useRef } from "react"
import AddSvg from "../../assets/icon/add.svg?react"
import RemoveSvg from "../../assets/icon/remove.svg?react"

export interface StepsTreeProps {
    root: StepInfoNode
    onAddChildren: (parent: StepInfoNode) => void
    onRemoveNode: (node: StepInfoNode) => void
    onStepSelected: (node: StepInfoNode) => void
}

export default function StepsTree({
    root,
    onAddChildren,
    onRemoveNode,
    onStepSelected,
}: StepsTreeProps) {
    return (
        <div className="steps-tree">
            <StepsTreeNode
                node={root}
                isNodeRoot={true}
                onStepSelected={onStepSelected}
                onAddChildren={onAddChildren}
                onRemoveNode={onRemoveNode}
            />
        </div>
    )
}

interface StepsTreeContentProps {
    node: StepInfoNode
    isNodeRoot: boolean
    onAddChildren: (parent: StepInfoNode) => void
    onRemoveNode: (node: StepInfoNode) => void
    onStepSelected: (node: StepInfoNode) => void
}

function StepsTreeNode({
    node,
    isNodeRoot,
    onAddChildren,
    onRemoveNode,
    onStepSelected,
}: StepsTreeContentProps) {
    const ref = useRef<HTMLDivElement>(null)
    return (
        <div ref={ref} className={"step-group"}>
            <StepPiece
                id={node.id}
                onAddButtonClicked={() => onAddChildren(node)}
                onRemoveButtonClicked={
                    isNodeRoot ? undefined : () => onRemoveNode(node)
                }
                onSelected={() => onStepSelected(node)}
            />
            {node.children.map((child) => (
                <BendableArrow
                    key={child.id}
                    area={ref}
                    startPos={"step-piece-" + node.id}
                    segments={[{ next: "step-piece-" + child.id }]}
                    onSegmentsChanges={() => {}}
                    forceStraight={true}
                    wavy={false}
                    //TODO remove magic constants
                    startRadius={10}
                    endRadius={10}
                />
            ))}
            <div className={"step-children"}>
                {node.children.map((child) => (
                    <StepsTreeNode
                        key={child.id}
                        isNodeRoot={false}
                        node={child}
                        onAddChildren={onAddChildren}
                        onRemoveNode={onRemoveNode}
                        onStepSelected={onStepSelected}
                    />
                ))}
            </div>
        </div>
    )
}

interface StepPieceProps {
    id: number
    onAddButtonClicked?: () => void
    onRemoveButtonClicked?: () => void
    onSelected: () => void
}

function StepPiece({
    id,
    onAddButtonClicked,
    onRemoveButtonClicked,
    onSelected,
}: StepPieceProps) {
    return (
        <div
            id={"step-piece-" + id}
            tabIndex={1}
            className={"step-piece"}
            onClick={onSelected}>
            <div className="step-piece-actions">
                {onAddButtonClicked && (
                    <AddSvg
                        onClick={() => onAddButtonClicked()}
                        className={"add-icon"}
                    />
                )}
                {onRemoveButtonClicked && (
                    <RemoveSvg
                        onClick={() => onRemoveButtonClicked()}
                        className={"remove-icon"}
                    />
                )}
            </div>
            <p>{id}</p>
        </div>
    )
}
